@@ -49,9 +49,10 @@ char	   *subarg = NULL;
 char		escape_buff[8192];
 
 void		do_substitute(char *subarg, dbhead * dbh);
-inline void strtoupper(char *string);
 
-inline void strtolower(char *string);
+static inline void strtoupper(char *string);
+static inline void strtolower(char *string);
+
 void		do_create(PGconn *, char *, dbhead *);
 void		do_inserts(PGconn *, char *, dbhead *);
 int			check_table(PGconn *, char *);
@@ -88,7 +89,7 @@ isinteger(char *buff)
 	return 1;
 }
 
-inline void
+static inline void
 strtoupper(char *string)
 {
 	while (*string != '\0')
@@ -98,7 +99,7 @@ strtoupper(char *string)
 	}
 }
 
-inline void
+static inline void
 strtolower(char *string)
 {
 	while (*string != '\0')
@@ -308,8 +309,8 @@ do_create(PGconn *conn, char *table, dbhead * dbh)
 				if (dbh->db_fields[i].db_flen > 1)
 				{
 					strcat(query, " varchar");
-					sprintf(t, "(%d)",
-							dbh->db_fields[i].db_flen);
+					snprintf(t, 20, "(%d)",
+							 dbh->db_fields[i].db_flen);
 					strcat(query, t);
 				}
 				else
@@ -361,7 +362,7 @@ do_inserts(PGconn *conn, char *table, dbhead * dbh)
 				result;
 	char	   *query,
 			   *foo;
-	char		pgdate[10];
+	char		pgdate[11];
 
 	if (verbose > 1)
 		printf("Inserting records\n");
@@ -467,9 +468,9 @@ do_inserts(PGconn *conn, char *table, dbhead * dbh)
 				{
 					if ((strlen(foo) == 8) && isinteger(foo))
 					{
-						sprintf(pgdate, "%c%c%c%c-%c%c-%c%c",
-								foo[0], foo[1], foo[2], foo[3],
-								foo[4], foo[5], foo[6], foo[7]);
+						snprintf(pgdate, 11, "%c%c%c%c-%c%c-%c%c",
+								 foo[0], foo[1], foo[2], foo[3],
+								 foo[4], foo[5], foo[6], foo[7]);
 						strcat(query, pgdate);
 					}
 					else
@@ -858,6 +859,10 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
+	PQexec(conn, "SET search_path = public");
+
+	PQexec(conn, "SET autocommit TO 'on'");
+
 /* Substitute field names */
 	do_substitute(subarg, dbh);
 
