@@ -4,12 +4,12 @@
  *	  Routines to determine which indices are usable for scanning a
  *	  given relation, and create IndexPaths accordingly.
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/optimizer/path/indxpath.c,v 1.112.2.2 2005/04/20 21:48:39 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/optimizer/path/indxpath.c,v 1.124.2.2 2005/04/20 21:48:31 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -21,6 +21,7 @@
 #include "access/nbtree.h"
 #include "catalog/catname.h"
 #include "catalog/pg_amop.h"
+#include "catalog/pg_namespace.h"
 #include "catalog/pg_operator.h"
 #include "executor/executor.h"
 #include "nodes/makefuncs.h"
@@ -96,7 +97,7 @@ static bool function_index_operand(Expr *funcOpnd, RelOptInfo *rel,
 static bool match_special_index_operator(Expr *clause, Oid opclass,
 							 bool indexkey_on_left);
 static List *prefix_quals(Var *leftop, Oid expr_op,
-			 char *prefix, Pattern_Prefix_Status pstatus);
+			 Const *prefix, Pattern_Prefix_Status pstatus);
 static List *network_prefix_quals(Var *leftop, Oid expr_op, Datum rightop);
 static Oid	find_operator(const char *opname, Oid datatype);
 static Datum string_to_datum(const char *str, Oid datatype);
@@ -795,7 +796,7 @@ match_clause_to_indexkey(RelOptInfo *rel,
 		 * Check for an indexqual that could be handled by a nestloop
 		 * join. We need the index key to be compared against an
 		 * expression that uses none of the indexed relation's vars and
-		 * contains no non-cachable functions.
+		 * contains no volatile functions.
 		 */
 		if (match_index_to_operand(indexkey, leftop, rel, index))
 		{
@@ -804,7 +805,7 @@ match_clause_to_indexkey(RelOptInfo *rel,
 
 			isIndexable =
 				!intMember(lfirsti(rel->relids), othervarnos) &&
-				!contain_noncachable_functions((Node *) rightop) &&
+				!contain_volatile_functions((Node *) rightop) &&
 				is_indexable_operator(clause, opclass, true);
 			freeList(othervarnos);
 			return isIndexable;
@@ -816,7 +817,7 @@ match_clause_to_indexkey(RelOptInfo *rel,
 
 			isIndexable =
 				!intMember(lfirsti(rel->relids), othervarnos) &&
-				!contain_noncachable_functions((Node *) leftop) &&
+				!contain_volatile_functions((Node *) leftop) &&
 				is_indexable_operator(clause, opclass, false);
 			freeList(othervarnos);
 			return isIndexable;
@@ -834,32 +835,15 @@ match_clause_to_indexkey(RelOptInfo *rel,
  * is whether the operator has a commutator operator that matches
  * the index's opclass.
  *
- * We try both the straightforward match and matches that rely on
- * recognizing binary-compatible datatypes.  For example, if we have
- * an expression like "oid = 123", the operator will be oideqint4,
- * which we need to replace with oideq in order to recognize it as
- * matching an oid_ops index on the oid field.	A variant case is where
- * the expression is like "oid::int4 = 123", where the given operator
- * will be int4eq and again we need to intuit that we want to use oideq.
- *
  * Returns the OID of the matching operator, or InvalidOid if no match.
- * Note that the returned OID will be different from the one in the given
- * expression if we used a binary-compatible substitution.	Also note that
- * if indexkey_on_left is FALSE (meaning we need to commute), the returned
- * OID is *not* commuted; it can be plugged directly into the given clause.
+ * (Formerly, this routine might return a binary-compatible operator
+ * rather than the original one, but that kluge is history.)
  */
 Oid
 indexable_operator(Expr *clause, Oid opclass, bool indexkey_on_left)
 {
 	Oid			expr_op = ((Oper *) clause->oper)->opno;
-	Oid			commuted_op,
-				new_op;
-	Operator	oldoptup;
-	Form_pg_operator oldopform;
-	char	   *opname;
-	Oid			ltype,
-				rtype,
-				indexkeytype;
+	Oid			commuted_op;
 
 	/* Get the commuted operator if necessary */
 	if (indexkey_on_left)
@@ -869,82 +853,10 @@ indexable_operator(Expr *clause, Oid opclass, bool indexkey_on_left)
 	if (commuted_op == InvalidOid)
 		return InvalidOid;
 
-	/* Done if the (commuted) operator is a member of the index's opclass */
+	/* OK if the (commuted) operator is a member of the index's opclass */
 	if (op_in_opclass(commuted_op, opclass))
 		return expr_op;
 
-	/*
-	 * Maybe the index uses a binary-compatible operator set.
-	 *
-	 * Get the nominal input types of the given operator and the actual type
-	 * (before binary-compatible relabeling) of the index key.
-	 */
-	oldoptup = SearchSysCache(OPEROID,
-							  ObjectIdGetDatum(expr_op),
-							  0, 0, 0);
-	if (!HeapTupleIsValid(oldoptup))
-		return InvalidOid;		/* probably can't happen */
-	oldopform = (Form_pg_operator) GETSTRUCT(oldoptup);
-	opname = pstrdup(NameStr(oldopform->oprname));
-	ltype = oldopform->oprleft;
-	rtype = oldopform->oprright;
-	ReleaseSysCache(oldoptup);
-
-	if (indexkey_on_left)
-	{
-		Node	   *leftop = (Node *) get_leftop(clause);
-
-		if (leftop && IsA(leftop, RelabelType))
-			leftop = ((RelabelType *) leftop)->arg;
-		indexkeytype = exprType(leftop);
-	}
-	else
-	{
-		Node	   *rightop = (Node *) get_rightop(clause);
-
-		if (rightop && IsA(rightop, RelabelType))
-			rightop = ((RelabelType *) rightop)->arg;
-		indexkeytype = exprType(rightop);
-	}
-
-	/*
-	 * Make sure we have different but binary-compatible types.
-	 */
-	if (ltype == indexkeytype && rtype == indexkeytype)
-		return InvalidOid;		/* no chance for a different operator */
-	if (ltype != indexkeytype && !IS_BINARY_COMPATIBLE(ltype, indexkeytype))
-		return InvalidOid;
-	if (rtype != indexkeytype && !IS_BINARY_COMPATIBLE(rtype, indexkeytype))
-		return InvalidOid;
-
-	/*
-	 * OK, look for operator of the same name with the indexkey's data
-	 * type. (In theory this might find a non-semantically-comparable
-	 * operator, but in practice that seems pretty unlikely for
-	 * binary-compatible types.)
-	 */
-	new_op = compatible_oper_opid(opname, indexkeytype, indexkeytype, true);
-
-	if (OidIsValid(new_op))
-	{
-		if (new_op != expr_op)
-		{
-			/*
-			 * OK, we found a binary-compatible operator of the same name;
-			 * now does it match the index?
-			 */
-			if (indexkey_on_left)
-				commuted_op = new_op;
-			else
-				commuted_op = get_commutator(new_op);
-			if (commuted_op == InvalidOid)
-				return InvalidOid;
-
-			if (op_in_opclass(commuted_op, opclass))
-				return new_op;
-		}
-	}
-
 	return InvalidOid;
 }
 
@@ -1167,7 +1079,7 @@ static const StrategyNumber
  *	  implies another.	A simple and general way is to see if they are
  *	  equal(); this works for any kind of expression.  (Actually, there
  *	  is an implied assumption that the functions in the expression are
- *	  cachable, ie dependent only on their input arguments --- but this
+ *	  immutable, ie dependent only on their input arguments --- but this
  *	  was checked for the predicate by CheckPredicate().)
  *
  *	  Our other way works only for (binary boolean) operators that are
@@ -1256,9 +1168,9 @@ pred_test_simple_clause(Expr *predicate, Node *clause)
 						   ObjectIdGetDatum(pred_op));
 
 	relation = heap_openr(AccessMethodOperatorRelationName, AccessShareLock);
-	scan = heap_beginscan(relation, false, SnapshotNow, 1, entry);
+	scan = heap_beginscan(relation, SnapshotNow, 1, entry);
 
-	while (HeapTupleIsValid(tuple = heap_getnext(scan, 0)))
+	while ((tuple = heap_getnext(scan, ForwardScanDirection)) != NULL)
 	{
 		aform = (Form_pg_amop) GETSTRUCT(tuple);
 		if (opclass_is_btree(aform->amopclaid))
@@ -1324,7 +1236,7 @@ pred_test_simple_clause(Expr *predicate, Node *clause)
 	if (!HeapTupleIsValid(tuple))
 	{
 		/* this probably shouldn't fail? */
-		elog(DEBUG, "pred_test_simple_clause: unknown test_op");
+		elog(DEBUG1, "pred_test_simple_clause: unknown test_op");
 		return false;
 	}
 	aform = (Form_pg_amop) GETSTRUCT(tuple);
@@ -1339,7 +1251,8 @@ pred_test_simple_clause(Expr *predicate, Node *clause)
 	 */
 	test_oper = makeOper(test_op,		/* opno */
 						 InvalidOid,	/* opid */
-						 BOOLOID);		/* opresulttype */
+						 BOOLOID,		/* opresulttype */
+						 false);	/* opretset */
 	replace_opid(test_oper);
 	test_expr = make_opclause(test_oper,
 							  (Var *) clause_const,
@@ -1352,7 +1265,7 @@ pred_test_simple_clause(Expr *predicate, Node *clause)
 
 	if (isNull)
 	{
-		elog(DEBUG, "pred_test_simple_clause: null test result");
+		elog(DEBUG1, "pred_test_simple_clause: null test result");
 		return false;
 	}
 	return DatumGetBool(test_result);
@@ -1682,10 +1595,9 @@ match_special_index_operator(Expr *clause, Oid opclass,
 	Var		   *leftop,
 			   *rightop;
 	Oid			expr_op;
-	Datum		constvalue;
-	char	   *patt;
-	char	   *prefix;
-	char	   *rest;
+	Const	   *patt = NULL;
+	Const	   *prefix = NULL;
+	Const	   *rest = NULL;
 
 	/*
 	 * Currently, all known special operators require the indexkey on the
@@ -1704,7 +1616,7 @@ match_special_index_operator(Expr *clause, Oid opclass,
 	if (!IsA(rightop, Const) ||
 		((Const *) rightop)->constisnull)
 		return false;
-	constvalue = ((Const *) rightop)->constvalue;
+	patt = (Const *) rightop;
 
 	switch (expr_op)
 	{
@@ -1712,68 +1624,45 @@ match_special_index_operator(Expr *clause, Oid opclass,
 		case OID_BPCHAR_LIKE_OP:
 		case OID_VARCHAR_LIKE_OP:
 		case OID_NAME_LIKE_OP:
+			/* the right-hand const is type text for all of these */
 			if (locale_is_like_safe())
-			{
-				/* the right-hand const is type text for all of these */
-				patt = DatumGetCString(DirectFunctionCall1(textout,
-														   constvalue));
 				isIndexable = pattern_fixed_prefix(patt, Pattern_Type_Like,
 								  &prefix, &rest) != Pattern_Prefix_None;
-				if (prefix)
-					pfree(prefix);
-				pfree(patt);
-			}
+			break;
+
+		case OID_BYTEA_LIKE_OP:
+			isIndexable = pattern_fixed_prefix(patt, Pattern_Type_Like,
+								  &prefix, &rest) != Pattern_Prefix_None;
 			break;
 
 		case OID_TEXT_ICLIKE_OP:
 		case OID_BPCHAR_ICLIKE_OP:
 		case OID_VARCHAR_ICLIKE_OP:
 		case OID_NAME_ICLIKE_OP:
+			/* the right-hand const is type text for all of these */
 			if (locale_is_like_safe())
-			{
-				/* the right-hand const is type text for all of these */
-				patt = DatumGetCString(DirectFunctionCall1(textout,
-														   constvalue));
 				isIndexable = pattern_fixed_prefix(patt, Pattern_Type_Like_IC,
 								  &prefix, &rest) != Pattern_Prefix_None;
-				if (prefix)
-					pfree(prefix);
-				pfree(patt);
-			}
 			break;
 
 		case OID_TEXT_REGEXEQ_OP:
 		case OID_BPCHAR_REGEXEQ_OP:
 		case OID_VARCHAR_REGEXEQ_OP:
 		case OID_NAME_REGEXEQ_OP:
+			/* the right-hand const is type text for all of these */
 			if (locale_is_like_safe())
-			{
-				/* the right-hand const is type text for all of these */
-				patt = DatumGetCString(DirectFunctionCall1(textout,
-														   constvalue));
 				isIndexable = pattern_fixed_prefix(patt, Pattern_Type_Regex,
 								  &prefix, &rest) != Pattern_Prefix_None;
-				if (prefix)
-					pfree(prefix);
-				pfree(patt);
-			}
 			break;
 
 		case OID_TEXT_ICREGEXEQ_OP:
 		case OID_BPCHAR_ICREGEXEQ_OP:
 		case OID_VARCHAR_ICREGEXEQ_OP:
 		case OID_NAME_ICREGEXEQ_OP:
+			/* the right-hand const is type text for all of these */
 			if (locale_is_like_safe())
-			{
-				/* the right-hand const is type text for all of these */
-				patt = DatumGetCString(DirectFunctionCall1(textout,
-														   constvalue));
 				isIndexable = pattern_fixed_prefix(patt, Pattern_Type_Regex_IC,
 								  &prefix, &rest) != Pattern_Prefix_None;
-				if (prefix)
-					pfree(prefix);
-				pfree(patt);
-			}
 			break;
 
 		case OID_INET_SUB_OP:
@@ -1784,6 +1673,12 @@ match_special_index_operator(Expr *clause, Oid opclass,
 			break;
 	}
 
+	if (prefix)
+	{
+		pfree(DatumGetPointer(prefix->constvalue));
+		pfree(prefix);
+	}
+
 	/* done if the expression doesn't look indexable */
 	if (!isIndexable)
 		return false;
@@ -1805,6 +1700,12 @@ match_special_index_operator(Expr *clause, Oid opclass,
 				isIndexable = false;
 			break;
 
+		case OID_BYTEA_LIKE_OP:
+			if (!op_in_opclass(find_operator(">=", BYTEAOID), opclass) ||
+				!op_in_opclass(find_operator("<", BYTEAOID), opclass))
+				isIndexable = false;
+			break;
+
 		case OID_BPCHAR_LIKE_OP:
 		case OID_BPCHAR_ICLIKE_OP:
 		case OID_BPCHAR_REGEXEQ_OP:
@@ -1874,10 +1775,9 @@ expand_indexqual_conditions(List *indexquals)
 		Var		   *leftop = get_leftop(clause);
 		Var		   *rightop = get_rightop(clause);
 		Oid			expr_op = ((Oper *) clause->oper)->opno;
-		Datum		constvalue;
-		char	   *patt;
-		char	   *prefix;
-		char	   *rest;
+		Const	   *patt = (Const *) rightop;
+		Const	   *prefix = NULL;
+		Const	   *rest = NULL;
 		Pattern_Prefix_Status pstatus;
 
 		switch (expr_op)
@@ -1892,18 +1792,12 @@ expand_indexqual_conditions(List *indexquals)
 			case OID_BPCHAR_LIKE_OP:
 			case OID_VARCHAR_LIKE_OP:
 			case OID_NAME_LIKE_OP:
-				/* the right-hand const is type text for all of these */
-				constvalue = ((Const *) rightop)->constvalue;
-				patt = DatumGetCString(DirectFunctionCall1(textout,
-														   constvalue));
+			case OID_BYTEA_LIKE_OP:
 				pstatus = pattern_fixed_prefix(patt, Pattern_Type_Like,
 											   &prefix, &rest);
 				resultquals = nconc(resultquals,
 									prefix_quals(leftop, expr_op,
 												 prefix, pstatus));
-				if (prefix)
-					pfree(prefix);
-				pfree(patt);
 				break;
 
 			case OID_TEXT_ICLIKE_OP:
@@ -1911,17 +1805,11 @@ expand_indexqual_conditions(List *indexquals)
 			case OID_VARCHAR_ICLIKE_OP:
 			case OID_NAME_ICLIKE_OP:
 				/* the right-hand const is type text for all of these */
-				constvalue = ((Const *) rightop)->constvalue;
-				patt = DatumGetCString(DirectFunctionCall1(textout,
-														   constvalue));
 				pstatus = pattern_fixed_prefix(patt, Pattern_Type_Like_IC,
 											   &prefix, &rest);
 				resultquals = nconc(resultquals,
 									prefix_quals(leftop, expr_op,
 												 prefix, pstatus));
-				if (prefix)
-					pfree(prefix);
-				pfree(patt);
 				break;
 
 			case OID_TEXT_REGEXEQ_OP:
@@ -1929,17 +1817,11 @@ expand_indexqual_conditions(List *indexquals)
 			case OID_VARCHAR_REGEXEQ_OP:
 			case OID_NAME_REGEXEQ_OP:
 				/* the right-hand const is type text for all of these */
-				constvalue = ((Const *) rightop)->constvalue;
-				patt = DatumGetCString(DirectFunctionCall1(textout,
-														   constvalue));
 				pstatus = pattern_fixed_prefix(patt, Pattern_Type_Regex,
 											   &prefix, &rest);
 				resultquals = nconc(resultquals,
 									prefix_quals(leftop, expr_op,
 												 prefix, pstatus));
-				if (prefix)
-					pfree(prefix);
-				pfree(patt);
 				break;
 
 			case OID_TEXT_ICREGEXEQ_OP:
@@ -1947,27 +1829,20 @@ expand_indexqual_conditions(List *indexquals)
 			case OID_VARCHAR_ICREGEXEQ_OP:
 			case OID_NAME_ICREGEXEQ_OP:
 				/* the right-hand const is type text for all of these */
-				constvalue = ((Const *) rightop)->constvalue;
-				patt = DatumGetCString(DirectFunctionCall1(textout,
-														   constvalue));
 				pstatus = pattern_fixed_prefix(patt, Pattern_Type_Regex_IC,
 											   &prefix, &rest);
 				resultquals = nconc(resultquals,
 									prefix_quals(leftop, expr_op,
 												 prefix, pstatus));
-				if (prefix)
-					pfree(prefix);
-				pfree(patt);
 				break;
 
 			case OID_INET_SUB_OP:
 			case OID_INET_SUBEQ_OP:
 			case OID_CIDR_SUB_OP:
 			case OID_CIDR_SUBEQ_OP:
-				constvalue = ((Const *) rightop)->constvalue;
 				resultquals = nconc(resultquals,
 									network_prefix_quals(leftop, expr_op,
-														 constvalue));
+													  patt->constvalue));
 				break;
 
 			default:
@@ -1983,19 +1858,20 @@ expand_indexqual_conditions(List *indexquals)
  * Given a fixed prefix that all the "leftop" values must have,
  * generate suitable indexqual condition(s).  expr_op is the original
  * LIKE or regex operator; we use it to deduce the appropriate comparison
- * operators.
+ * operators and operand datatypes.
  */
 static List *
 prefix_quals(Var *leftop, Oid expr_op,
-			 char *prefix, Pattern_Prefix_Status pstatus)
+			 Const *prefix_const, Pattern_Prefix_Status pstatus)
 {
 	List	   *result;
 	Oid			datatype;
 	Oid			oproid;
+	char	   *prefix;
 	Const	   *con;
 	Oper	   *op;
 	Expr	   *expr;
-	char	   *greaterstr;
+	Const	   *greaterstr = NULL;
 
 	Assert(pstatus != Pattern_Prefix_None);
 
@@ -2008,6 +1884,10 @@ prefix_quals(Var *leftop, Oid expr_op,
 			datatype = TEXTOID;
 			break;
 
+		case OID_BYTEA_LIKE_OP:
+			datatype = BYTEAOID;
+			break;
+
 		case OID_BPCHAR_LIKE_OP:
 		case OID_BPCHAR_ICLIKE_OP:
 		case OID_BPCHAR_REGEXEQ_OP:
@@ -2034,6 +1914,14 @@ prefix_quals(Var *leftop, Oid expr_op,
 			return NIL;
 	}
 
+	/* Prefix constant is text for all except BYTEA_LIKE */
+	if (datatype != BYTEAOID)
+		prefix = DatumGetCString(DirectFunctionCall1(textout,
+													 prefix_const->constvalue));
+	else
+		prefix = DatumGetCString(DirectFunctionCall1(byteaout,
+													 prefix_const->constvalue));
+
 	/*
 	 * If we found an exact-match pattern, generate an "=" indexqual.
 	 */
@@ -2043,7 +1931,7 @@ prefix_quals(Var *leftop, Oid expr_op,
 		if (oproid == InvalidOid)
 			elog(ERROR, "prefix_quals: no = operator for type %u", datatype);
 		con = string_to_const(prefix, datatype);
-		op = makeOper(oproid, InvalidOid, BOOLOID);
+		op = makeOper(oproid, InvalidOid, BOOLOID, false);
 		expr = make_opclause(op, leftop, (Var *) con);
 		result = makeList1(expr);
 		return result;
@@ -2058,7 +1946,7 @@ prefix_quals(Var *leftop, Oid expr_op,
 	if (oproid == InvalidOid)
 		elog(ERROR, "prefix_quals: no >= operator for type %u", datatype);
 	con = string_to_const(prefix, datatype);
-	op = makeOper(oproid, InvalidOid, BOOLOID);
+	op = makeOper(oproid, InvalidOid, BOOLOID, false);
 	expr = make_opclause(op, leftop, (Var *) con);
 	result = makeList1(expr);
 
@@ -2067,17 +1955,15 @@ prefix_quals(Var *leftop, Oid expr_op,
 	 * "x < greaterstr".
 	 *-------
 	 */
-	greaterstr = make_greater_string(prefix, datatype);
+	greaterstr = make_greater_string(con);
 	if (greaterstr)
 	{
 		oproid = find_operator("<", datatype);
 		if (oproid == InvalidOid)
 			elog(ERROR, "prefix_quals: no < operator for type %u", datatype);
-		con = string_to_const(greaterstr, datatype);
-		op = makeOper(oproid, InvalidOid, BOOLOID);
-		expr = make_opclause(op, leftop, (Var *) con);
+		op = makeOper(oproid, InvalidOid, BOOLOID, false);
+		expr = make_opclause(op, leftop, (Var *) greaterstr);
 		result = lappend(result, expr);
-		pfree(greaterstr);
 	}
 
 	return result;
@@ -2139,7 +2025,7 @@ network_prefix_quals(Var *leftop, Oid expr_op, Datum rightop)
 
 	opr1right = network_scan_first(rightop);
 
-	op = makeOper(opr1oid, InvalidOid, BOOLOID);
+	op = makeOper(opr1oid, InvalidOid, BOOLOID, false);
 	expr = make_opclause(op, leftop,
 						 (Var *) makeConst(datatype, -1, opr1right,
 										   false, false, false, false));
@@ -2154,7 +2040,7 @@ network_prefix_quals(Var *leftop, Oid expr_op, Datum rightop)
 
 	opr2right = network_scan_last(rightop);
 
-	op = makeOper(opr2oid, InvalidOid, BOOLOID);
+	op = makeOper(opr2oid, InvalidOid, BOOLOID, false);
 	expr = make_opclause(op, leftop,
 						 (Var *) makeConst(datatype, -1, opr2right,
 										   false, false, false, false));
@@ -2168,14 +2054,15 @@ network_prefix_quals(Var *leftop, Oid expr_op, Datum rightop)
  */
 
 /* See if there is a binary op of the given name for the given datatype */
+/* NB: we assume that only built-in system operators are searched for */
 static Oid
 find_operator(const char *opname, Oid datatype)
 {
-	return GetSysCacheOid(OPERNAME,
+	return GetSysCacheOid(OPERNAMENSP,
 						  PointerGetDatum(opname),
 						  ObjectIdGetDatum(datatype),
 						  ObjectIdGetDatum(datatype),
-						  CharGetDatum('b'));
+						  ObjectIdGetDatum(PG_CATALOG_NAMESPACE));
 }
 
 /*
@@ -2192,6 +2079,8 @@ string_to_datum(const char *str, Oid datatype)
 	 */
 	if (datatype == NAMEOID)
 		return DirectFunctionCall1(namein, CStringGetDatum(str));
+	else if (datatype == BYTEAOID)
+		return DirectFunctionCall1(byteain, CStringGetDatum(str));
 	else
 		return DirectFunctionCall1(textin, CStringGetDatum(str));
 }
